@@ -1,5 +1,5 @@
 """
-Method agnostic utility functions for linear progamming
+Method agnostic utility functions for linear programming
 """
 
 import numpy as np
@@ -51,6 +51,28 @@ _LPProblem.__doc__ = \
         the optimization algorithm. This argument is currently used only by the
         'revised simplex' method, and can only be used if `x0` represents a
         basic feasible solution.
+    integrality : 1-D array or int, optional
+        Indicates the type of integrality constraint on each decision variable.
+
+        ``0`` : Continuous variable; no integrality constraint.
+
+        ``1`` : Integer variable; decision variable must be an integer
+        within `bounds`.
+
+        ``2`` : Semi-continuous variable; decision variable must be within
+        `bounds` or take value ``0``.
+
+        ``3`` : Semi-integer variable; decision variable must be an integer
+        within `bounds` or take value ``0``.
+
+        By default, all variables are continuous.
+
+        For mixed integrality constraints, supply an array of shape `c.shape`.
+        To infer a constraint on each decision variable from shorter inputs,
+        the argument will be broadcast to `c.shape` using `np.broadcast_to`.
+
+        This argument is currently used only by the ``'highs'`` method and
+        ignored otherwise.
 
     Notes
     -----
@@ -185,7 +207,7 @@ def _format_b_constraints(b):
     if b is None:
         return np.array([], dtype=float)
     b = np.array(b, dtype=float, copy=True).squeeze()
-    return b if b.size != 1 else b.reshape((-1))
+    return b if b.size != 1 else b.reshape(-1)
 
 
 def _clean_inputs(lp):
@@ -273,14 +295,14 @@ def _clean_inputs(lp):
     else:
         # If c is a single value, convert it to a 1-D array.
         if c.size == 1:
-            c = c.reshape((-1))
+            c = c.reshape(-1)
 
         n_x = len(c)
         if n_x == 0 or len(c.shape) != 1:
             raise ValueError(
                 "Invalid input for linprog: c must be a 1-D array and must "
                 "not have more than one non-singleton dimension")
-        if not(np.isfinite(c).all()):
+        if not np.isfinite(c).all():
             raise ValueError(
                 "Invalid input for linprog: c must not contain values "
                 "inf, nan, or None")
@@ -319,7 +341,7 @@ def _clean_inputs(lp):
                 "must not have more than one non-singleton dimension and "
                 "the number of rows in A_ub must equal the number of values "
                 "in b_ub")
-        if not(np.isfinite(b_ub).all()):
+        if not np.isfinite(b_ub).all():
             raise ValueError(
                 "Invalid input for linprog: b_ub must not contain values "
                 "inf, nan, or None")
@@ -358,7 +380,7 @@ def _clean_inputs(lp):
                 "must not have more than one non-singleton dimension and "
                 "the number of rows in A_eq must equal the number of values "
                 "in b_eq")
-        if not(np.isfinite(b_eq).all()):
+        if not np.isfinite(b_eq).all():
             raise ValueError(
                 "Invalid input for linprog: b_eq must not contain values "
                 "inf, nan, or None")
@@ -373,7 +395,7 @@ def _clean_inputs(lp):
                 "Invalid input for linprog: x0 must be a 1-D array of "
                 "numerical coefficients") from e
         if x0.ndim == 0:
-            x0 = x0.reshape((-1))
+            x0 = x0.reshape(-1)
         if len(x0) == 0 or x0.ndim != 1:
             raise ValueError(
                 "Invalid input for linprog: x0 should be a 1-D array; it "
@@ -422,7 +444,7 @@ def _clean_inputs(lp):
         # Do not try to handle multidimensional bounds input
         raise ValueError(
             "Invalid input for linprog: provide a 2-D array for bounds, "
-            "not a {:d}-D array.".format(len(bsh)))
+            f"not a {len(bsh):d}-D array.")
     elif np.all(bsh == (n_x, 2)):
         # Regular N x 2 array
         bounds_clean = bounds_conv
@@ -434,12 +456,12 @@ def _clean_inputs(lp):
     elif np.all(bsh == (2, n_x)):
         # Reject a 2 x N array
         raise ValueError(
-            "Invalid input for linprog: provide a {:d} x 2 array for bounds, "
-            "not a 2 x {:d} array.".format(n_x, n_x))
+            f"Invalid input for linprog: provide a {n_x:d} x 2 array for bounds, "
+            f"not a 2 x {n_x:d} array.")
     else:
         raise ValueError(
             "Invalid input for linprog: unable to interpret bounds with this "
-            "dimension tuple: {0}.".format(bsh))
+            f"dimension tuple: {bsh}.")
 
     # The process above creates nan-s where the input specified None
     # Convert the nan-s in the 1st column to -np.inf and in the 2nd column
@@ -1193,7 +1215,7 @@ def _get_Abc(lp, c0):
     if sparse:
         b = b.reshape(-1, 1)
         A = A.tocsc()
-        b -= (A[:, i_shift] * sps.diags(lb_shift)).sum(axis=1)
+        b -= (A[:, i_shift] @ sps.diags(lb_shift)).sum(axis=1)
         b = b.ravel()
     else:
         b -= (A[:, i_shift] * lb_shift).sum(axis=1)
@@ -1227,7 +1249,7 @@ def _autoscale(A, b, c, x0):
             R = R.toarray().flatten()
         R[R == 0] = 1
         R = 1/_round_to_power_of_two(R)
-        A = sps.diags(R)*A if sps.issparse(A) else A*R.reshape(m, 1)
+        A = sps.diags(R)@A if sps.issparse(A) else A*R.reshape(m, 1)
         b = b*R
 
         C = np.max(np.abs(A), axis=0)
@@ -1235,7 +1257,7 @@ def _autoscale(A, b, c, x0):
             C = C.toarray().flatten()
         C[C == 0] = 1
         C = 1/_round_to_power_of_two(C)
-        A = A*sps.diags(C) if sps.issparse(A) else A*C
+        A = A@sps.diags(C) if sps.issparse(A) else A*C
         c = c*C
 
     b_scale = np.max(np.abs(b)) if b.size > 0 else 1
@@ -1287,8 +1309,8 @@ def _display_summary(message, status, fun, iteration):
     """
     print(message)
     if status in (0, 1):
-        print("         Current function value: {0: <12.6f}".format(fun))
-    print("         Iterations: {0:d}".format(iteration))
+        print(f"         Current function value: {fun: <12.6f}")
+    print(f"         Iterations: {iteration:d}")
 
 
 def _postsolve(x, postsolve_args, complete=False):
@@ -1394,7 +1416,8 @@ def _postsolve(x, postsolve_args, complete=False):
     return x, fun, slack, con
 
 
-def _check_result(x, fun, status, slack, con, bounds, tol, message):
+def _check_result(x, fun, status, slack, con, bounds, tol, message,
+                  integrality):
     """
     Check the validity of the provided solution.
 
@@ -1466,7 +1489,13 @@ def _check_result(x, fun, status, slack, con, bounds, tol, message):
     if contains_nans:
         is_feasible = False
     else:
-        invalid_bounds = (x < bounds[:, 0] - tol).any() or (x > bounds[:, 1] + tol).any()
+        if integrality is None:
+            integrality = 0
+        valid_bounds = (x >= bounds[:, 0] - tol) & (x <= bounds[:, 1] + tol)
+        # When integrality is 2 or 3, x must be within bounds OR take value 0
+        valid_bounds |= (integrality > 1) & np.isclose(x, 0, atol=tol)
+        invalid_bounds = not np.all(valid_bounds)
+
         invalid_slack = status != 3 and (slack < -tol).any()
         invalid_con = status != 3 and (np.abs(con) > tol).any()
         is_feasible = not (invalid_bounds or invalid_slack or invalid_con)
@@ -1474,7 +1503,7 @@ def _check_result(x, fun, status, slack, con, bounds, tol, message):
     if status == 0 and not is_feasible:
         status = 4
         message = ("The solution does not satisfy the constraints within the "
-                   "required tolerance of " + "{:.2E}".format(tol) + ", yet "
+                   "required tolerance of " + f"{tol:.2E}" + ", yet "
                    "no errors were raised and there is no certificate of "
                    "infeasibility or unboundedness. Check whether "
                    "the slack and constraint residuals are acceptable; "
